==> brainfuck-1.0.0-sources.jar: de/fxnn/brainfuck/tape/AbstractInfiniteTape.java <==
package de.fxnn.brainfuck.tape;

import java.util.Optional;

import javax.annotation.Nonnull;

public abstract class AbstractInfiniteTape<T> implements Tape<T> {

  private TapeSegment<T> currentSegment;

  public AbstractInfiniteTape() {
    currentSegment = createSegment();
  }

  @Override
  public void moveForward() throws OutOfTapeBoundsException {
    try {
      currentSegment.moveForward();
    } catch (OutOfTapeBoundsException ex) {
      currentSegment = getOrCreateNextSegment().atBeginning();
    }
  }

  protected TapeSegment<T> getOrCreateNextSegment() {
    return currentSegment.getNextSegment().orElseGet(this::createNextSegment);
  }

  protected TapeSegment<T> createNextSegment() {
    TapeSegment<T> nextSegment = createSegment();
    nextSegment.setPreviousSegment(Optional.of(currentSegment));
    currentSegment.setNextSegment(Optional.of(nextSegment));

    return nextSegment;
  }

  @Override
  public void moveBackward() throws OutOfTapeBoundsException {
    try {
      currentSegment.moveBackward();
    } catch (OutOfTapeBoundsException ex) {
      currentSegment = getOrCreatePreviousSegment().atEnd();
    }
  }

  public TapeSegment<T> getOrCreatePreviousSegment() {
    return createSegment().getPreviousSegment().orElseGet(this::createPreviousSegment);
  }

  private TapeSegment<T> createPreviousSegment() {
    TapeSegment<T> previousSegment = createSegment();
    previousSegment.setNextSegment(Optional.of(currentSegment));
    currentSegment.setPreviousSegment(Optional.of(previousSegment));

    return previousSegment;
  }

  protected abstract TapeSegment<T> createSegment();

  protected void write(@Nonnull T value) {
    currentSegment.write(value);
  }

  protected T read() {
    return currentSegment.read();
  }

}

==> brainfuck-1.3.0-sources.jar: de/fxnn/brainfuck/tape/AbstractInfiniteTape.java <==
package de.fxnn.brainfuck.tape;

import javax.annotation.Nonnull;

public abstract class AbstractInfiniteTape<T> implements Tape<T> {

  private TapeSegment<T> currentSegment;

  public AbstractInfiniteTape() {
    currentSegment = createSegment();
  }

  @Override
  public void moveForward() throws OutOfTapeBoundsException {
    try {
      currentSegment.moveForward();
    } catch (OutOfTapeBoundsException ex) {
      currentSegment = getOrCreateNextSegment().atBeginning();
    }
  }

  protected TapeSegment<T> getOrCreateNextSegment() {
    TapeSegment<T> nextSegment = currentSegment.getNextSegment();
    if (nextSegment != null) {
      return nextSegment;
    }

    return createNextSegment();
  }

  protected TapeSegment<T> createNextSegment() {
    TapeSegment<T> nextSegment = createSegment();
    nextSegment.setPreviousSegment(currentSegment);
    currentSegment.setNextSegment(nextSegment);

    return nextSegment;
  }

  @Override
  public void moveBackward() throws OutOfTapeBoundsException {
    try {
      currentSegment.moveBackward();
    } catch (OutOfTapeBoundsException ex) {
      currentSegment = getOrCreatePreviousSegment().atEnd();
    }
  }

  public TapeSegment<T> getOrCreatePreviousSegment() {
    TapeSegment<T> previousSegment = createSegment().getPreviousSegment();
    if (previousSegment != null) {
      return previousSegment;
    }

    return createPreviousSegment();
  }

  private TapeSegment<T> createPreviousSegment() {
    TapeSegment<T> previousSegment = createSegment();
    previousSegment.setNextSegment(currentSegment);
    currentSegment.setPreviousSegment(previousSegment);

    return previousSegment;
  }

  protected abstract TapeSegment<T> createSegment();

  protected void write(@Nonnull T value) {
    currentSegment.write(value);
  }

  protected T read() {
    return currentSegment.read();
  }

}
